/*****************************************************************************

  Licensed to Accellera Systems Initiative Inc. (Accellera) under one or
  more contributor license agreements.  See the NOTICE file distributed
  with this work for additional information regarding copyright ownership.
  Accellera licenses this file to you under the Apache License, Version 2.0
  (the "License"); you may not use this file except in compliance with the
  License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
  implied.  See the License for the specific language governing
  permissions and limitations under the License.

 *****************************************************************************/

// user's guide amplifier example as template class

#include <systemc-ams>
#include "test_utilities.h"

#include <cmath>
#include <complex>

SCA_TDF_MODULE(sin_src)
{
  sca_tdf::sca_out<double> out; // output port

  sin_src( sc_core::sc_module_name nm, double ampl_= 1.0, double freq_ = 1.0e3,
           sca_core::sca_time Tm_ = sca_core::sca_time(0.125, sc_core::SC_MS) )
  : out("out"), ampl(ampl_), freq(freq_), Tm(Tm_)
  {}

  void set_attributes()
  {
    out.set_timestep(Tm);
  }

  void processing()
  {
    double t = get_time().to_seconds(); // actual time
    out.write(ampl * std::sin( 2.0 * M_PI * freq * t ) ) ;
  }

 private:
  double ampl; // amplitude
  double freq; // frequency
  sca_core::sca_time Tm; // module time step
};

template <class T>
SCA_TDF_MODULE(amplifier)
{
  sca_tdf::sca_in<T>  in;      
  sca_tdf::sca_out<T> out;

  amplifier( sc_core::sc_module_name, double gain = 1.0, double iip3 = 1e-3 )
  : in("in"), out("out"), a1( gain ), a3( -4 / 3 * (gain / std::pow(iip3, 2)) ) {}

  void processing()
  { 
    out.write( a1 * in.read() + a3 * std::pow(in.read(), 3) ); 
  }
  
 private:
  double a1, a3; 
};

int sc_main( int, char*[] )
{
  TEST_LABEL_START;

  sca_tdf::sca_signal<double> sig1, sig2;
  
  sin_src src("src");
    src.out(sig1);

  amplifier<double> amp("amp");
    amp.in(sig1);
    amp.out(sig2);
    
  sc_core::sc_start(10, sc_core::SC_MS);
  
  TEST_LABEL_END;
    
  return 0;
}
  
