/*****************************************************************************

  Licensed to Accellera Systems Initiative Inc. (Accellera) under one or
  more contributor license agreements.  See the NOTICE file distributed
  with this work for additional information regarding copyright ownership.
  Accellera licenses this file to you under the Apache License, Version 2.0
  (the "License"); you may not use this file except in compliance with the
  License.  You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
  implied.  See the License for the specific language governing
  permissions and limitations under the License.

 *****************************************************************************/

//  set_sink_change_sink_od.cpp
//    -- Test TDF-DE out port set_timestep() in the context of change_attributes()
//       after set_timestep() on module in set_attributes().

#include <systemc-ams>
#include "test_utilities.h"

SCA_TDF_MODULE(src)
{
  sca_tdf::sca_out<int> out;

  SCA_CTOR(src)
  : out("out"), i(0)
  {}

  void set_attributes()
  {
    accept_attribute_changes();
  }

  void processing()
  {
    out.write(++i);
  }

private:
  int i;
};


SCA_TDF_MODULE(sink)
{
  sca_tdf::sca_in<int> in;
  sca_tdf::sca_de::sca_out<int> outd;

  SCA_CTOR(sink)
  : in("in"), outd("outd"), i(0), period(10)
  {}

  void set_attributes()
  {
    does_attribute_changes();
    accept_attribute_changes();
    this->set_timestep( 1, sc_core::SC_US );
  }

  void processing()
  {
    i = in.read();
    outd.write(i);
    std::cout << this->name() << " @ " << this->get_time()
              << ": " << i << std::endl;
  }

  void change_attributes()
  {
    if ( !(i % period) )
    {
      outd.set_timestep( (i / period) + 1, sc_core::SC_US );
    }
  }

private:
  int i, period;
};

int sc_main(int argc, char* argv[])
{
  using namespace sc_core;

  TEST_LABEL_START;

  sca_tdf::sca_signal <int> sig1("sig1");
  sc_signal<int> sig2("sig2");

  src src1("src1");
  src1.out(sig1);

  sink sink1("sink1");
  sink1.in(sig1);
  sink1.outd(sig2);

  sc_time tstop(100.0, SC_US);
  
  try
  { 
    sc_start(tstop);
  }
  catch (const std::exception& e)
  {
    std::cerr << e.what() << std::endl;
  }

  TEST_LABEL_END;
  sc_stop();
  return sc_report_handler::get_count(SC_ERROR);
}

// vim: expandtab : tabstop=2 : softtabstop=2 : shiftwidth=2
